/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Sonatype/repoServer/NugetUploadPanel', function() {

  Sonatype.repoServer.NugetUploadPanel = function(config) {
    var config = config || {};
    var defaultConfig = {};
    Ext.apply(this, config, defaultConfig);

    this.fileInput = null;

    if (this.extraItems == undefined)
    {
      this.extraItems = {};
    }

    Sonatype.repoServer.NugetUploadPanel.superclass.constructor.call(this, {
          region : 'center',
          id : (this.id) ? this.id : 'nugetUploadFormId',
          trackResetOnLoad : true,
          autoScroll : true,
          bodyStyle : 'overflow:auto;',
          border : false,
          frame : true,
          collapsible : false,
          collapsed : false,
          width : '100%',
          fileUpload : true,
          layoutConfig : {
            labelSeparator : ''
          },
          items : [{
                xtype : 'hidden',
                name : 'r',
                value : this.payload.id
              }, {
                xtype : 'fieldset',
                id : 'packages-upload-fieldset',
                checkboxToggle : false,
                title : 'Select Package(s) for Upload',
                collapsible : false,
                autoHeight : true,
                style : 'margin-right:7px;width:95%;',
                items : [{
                      hideLabel : true,
                      xtype : 'browsebutton',
                      text : 'Browse...',
                      style : 'margin-bottom: 5px;',
                      uploadPanel : this,
                      name : 'uploadPackageButton',
                      handler : function(b) {
                        b.uploadPanel.fileInput = b.detachInputFile();
                        var filename = b.uploadPanel.fileInput.getValue();
                        b.uploadPanel.updateFilename(b.uploadPanel, filename);
                      }
                    }, {
                      xtype : 'textfield',
                      fieldLabel : 'Filename',
                      name : 'filenameField',
                      readOnly : true,
                      width : '95%',
                      allowBlank : true
                    }, {
                      xtype : 'button',
                      id : 'add-button',
                      text : 'Add Package',
                      handler : this.addPackage,
                      scope : this,
                      disabled : true
                    }, {
                      xtype : 'panel',
                      layout : 'column',
                      autoHeight : true,
                      style : 'padding-top: 5px; padding-bottom: 5px;',
                      items : [{
                            xtype : 'treepanel',
                            name : 'package-list',
                            title : 'Packages',
                            border : true,
                            bodyBorder : true,
                            bodyStyle : 'background-color:#FFFFFF; border: 1px solid #B5B8C8',
                            style : 'padding: 0px 10px 0px 105px',
                            width : 500,
                            height : 100,
                            animate : true,
                            lines : false,
                            autoScroll : true,
                            containerScroll : true,
                            rootVisible : false,
                            ddScroll : true,
                            enableDD : true,
                            root : new Ext.tree.TreeNode({
                                  text : 'root',
                                  draggable : false
                                }),
                            invalidText : 'Add one or more packages',
                            validate : function() {
                              return this.find('name', 'package-list')[0].root.childNodes.length > 0;
                            },
                            invalid : false,
                            listeners : {
                              'append' : {
                                fn : function(tree, parentNode, insertedNode, i) {
                                  if (tree.invalid)
                                  {
                                    this.clearInvalid();
                                  }
                                },
                                scope : this
                              },
                              'remove' : {
                                fn : function(tree, parentNode, removedNode) {
                                  if (tree.root.childNodes.length < 1 && tree.required)
                                  {
                                    this.markTreeInvalid(null);
                                  }
                                  else
                                  {
                                    this.clearInvalid();
                                  }
                                },
                                scope : this
                              }
                            }
                          }, {
                            xtype : 'panel',
                            width : 120,
                            items : [{
                                  xtype : 'button',
                                  text : 'Remove',
                                  minWidth : 100,
                                  id : 'button-remove',
                                  handler : this.removePackage,
                                  scope : this
                                }, {
                                  xtype : 'button',
                                  text : 'Remove All',
                                  style : 'margin-top: 5px',
                                  minWidth : 100,
                                  id : 'button-remove-all',
                                  handler : this.removeAllPackages,
                                  scope : this
                                }]
                          }]
                    }, this.extraItems, {
                      xtype : 'panel',
                      id : 'end-button-card-panel',
                      header : false,
                      deferredRender : false,
                      autoScroll : false,
                      layout : 'fit',
                      buttonAlign : 'center',
                      frame : false,
                      items : [{}],
                      buttons : [{
                            xtype : 'button',
                            id : 'upload-button',
                            text : 'Upload Package(s)',
                            handler : this.uploadPackages,
                            scope : this
                          }, {
                            xtype : 'button',
                            id : 'reset-all-button',
                            text : 'Reset',
                            handler : this.resetFields,
                            scope : this
                          }]
                    }]
              }]
        });
  };

  Ext.extend(Sonatype.repoServer.NugetUploadPanel, Ext.FormPanel, {
        clearInvalid : function() {
          var tree = this.find('name', 'package-list')[0];
          if (tree.invalid)
          {
            // remove error messaging
            tree.getEl().child('.x-panel-body').setStyle({
                  'background-color' : '#FFFFFF',
                  border : '1px solid #B5B8C8'
                });
            Ext.form.Field.msgFx['normal'].hide(tree.errorEl, tree);
          }
        },
        markTreeInvalid : function(errortext) {
          var tree = this.find('name', 'package-list')[0];
          var elp = tree.getEl();

          if (!tree.errorEl)
          {
            tree.errorEl = elp.createChild({
                  cls : 'x-form-invalid-msg'
                });
            tree.errorEl.setWidth(elp.getWidth(true)); // note removed -20 like
            // on form fields
          }
          tree.invalid = true;
          var oldErrorText = tree.invalidText;
          if (errortext)
          {
            tree.invalidText = errortext;
          }
          tree.errorEl.update(tree.invalidText);
          tree.invalidText = oldErrorText;
          elp.child('.x-panel-body').setStyle({
                'background-color' : '#fee',
                border : '1px solid #dd7870'
              });
          Ext.form.Field.msgFx['normal'].show(tree.errorEl, tree);
        },
        resetFields : function() {
          // reset the package panels
          var filenameField = this.find('name', 'filenameField')[0];
          filenameField.reset();
          var addPackageBtn = this.find('id', 'add-button')[0];
          addPackageBtn.setDisabled(true);

          // clear the packages fields
          this.removeAllPackages();

          this.fileInput = null;

          var desc = this.find('name', 'description')[0];
          if (desc)
          {
            desc.reset();
          }
        },

        addPackage : function() {
          var treePanel = this.find('name', 'package-list')[0];
          var filenameField = this.find('name', 'filenameField')[0];
          var nodeText = filenameField.getValue();

          if (this.fileInput != null)
          {
            treePanel.root.appendChild(new Ext.tree.TreeNode({
                  id : filenameField.getValue(),
                  text : nodeText,
                  payload : {
                    id : filenameField.getValue(),
                    filename : filenameField.getValue(),
                    fileInput : this.fileInput
                  },
                  allowChildren : false,
                  draggable : false,
                  leaf : true,
                  icon : Sonatype.config.extPath + '/resources/images/default/tree/leaf.gif'
                }));
          }
          filenameField.setValue('');
          this.fileInput = null;
          this.find('id', 'add-button')[0].setDisabled(true);
        },
        removePackage : function() {
          var treePanel = this.find('name', 'package-list')[0];

          var selectedNode = treePanel.getSelectionModel().getSelectedNode();
          if (selectedNode)
          {
            treePanel.root.removeChild(selectedNode);
          }
        },
        removeAllPackages : function() {
          var treePanel = this.find('name', 'package-list')[0];
          var treeRoot = treePanel.root;

          while (treeRoot.lastChild)
          {
            treeRoot.removeChild(treeRoot.lastChild);
          }
        },
        updateFilename : function(uploadPanel, filename) {
          var filenameField = uploadPanel.find('name', 'filenameField')[0];
          if (filename)
          {
            filenameField.setValue(filename);
          }
          else
          {
            if (!(filename = filenameField.getValue()))
            {
              return;
            }
          }
          uploadPanel.find('id', 'add-button')[0].setDisabled(false);
        },
        uploadPackages : function() {
          var tree = this.find('name', 'package-list')[0];
          if (!tree.validate.call(this))
          {
            this.markTreeInvalid(null);
          }
          else if (this.form.isValid())
          {
            this.doUpload();
          }
        },
        doUpload : function() {
          var treePanel = this.find('name', 'package-list')[0];
          var hasPackages = treePanel.root.childNodes != null && treePanel.root.childNodes.length > 0;
          if (!hasPackages)
          {
            Sonatype.MessageBox.show({
                  title : 'No Packages Selected',
                  msg : 'The Packages list must contain at least one package to upload.',
                  buttons : Sonatype.MessageBox.OK,
                  icon : Sonatype.MessageBox.ERROR
                });
            return;
          }

          Sonatype.MessageBox.wait('Uploading ...');

          if (hasPackages)
          {
            this.currentChildNode = 0;
            this.createUploadForm(treePanel,
                treePanel.root.childNodes[this.currentChildNode].attributes.payload.fileInput,
                this.currentChildNode == (treePanel.root.childNodes.length - 1));
          }
        },
        createUploadForm : function(treePanel, fileInput, lastItem) {
          var repoId = this.payload.id;
          repoId = repoId.substring(repoId.lastIndexOf('/') + 1);

          var repoTag = {
            tag : 'input',
            type : 'hidden',
            name : 'r',
            value : repoId
          };

          var tmpForm = Ext.getBody().createChild({
                tag : 'form',
                cls : 'x-hidden',
                id : Ext.id(),
                children : [repoTag]
              });

          if (fileInput)
          {
            fileInput.appendTo(tmpForm);
          }

          Ext.Ajax.request({
                url : Sonatype.config.servicePath + '/artifact/nuget/content',
                form : tmpForm,
                isUpload : true,
                cbPassThru : {
                  treePanel : treePanel
                },
                callback : function(options, success, response) {
                  tmpForm.remove();

                  // This is a hack to get around the fact that upload submit
                  // always returns
                  // success = true
                  if (response.responseText.toLowerCase().indexOf('<error>') == -1)
                  {
                    if (lastItem)
                    {
                      Sonatype.MessageBox.show({
                            title : 'Upload Complete',
                            msg : 'Package upload finished successfully',
                            buttons : Sonatype.MessageBox.OK,
                            icon : Sonatype.MessageBox.INFO
                          });
                      this.resetFields();
                    }
                    else
                    {
                      this.currentChildNode++;
                      var treePanel = options.cbPassThru.treePanel;
                      if (this.currentChildNode < treePanel.root.childNodes.length)
                      {
                        this.createUploadForm(treePanel,
                            treePanel.root.childNodes[this.currentChildNode].attributes.payload.fileInput,
                            this.currentChildNode == treePanel.root.childNodes.length - 1);
                      }
                    }
                  }
                  else
                  {
                    var s = 'Package upload failed.<br />';
                    var r = response.responseText;
                    var n1 = r.toLowerCase().indexOf('<error>') + 7;
                    var n2 = r.toLowerCase().indexOf('</error>');
                    if (n2 > n1)
                    {
                      s += r.substring(n1, n2);
                    }
                    else
                    {
                      s += 'Check Nexus logs for more information.';
                    }
                    Sonatype.MessageBox.show({
                          title : 'Upload Failed',
                          msg : s,
                          buttons : Sonatype.MessageBox.OK,
                          icon : Sonatype.MessageBox.ERROR
                        });
                  }
                },
                scope : this
              });
        }
      });

  Sonatype.Events.addListener('repositoryViewInit', function(cardPanel, rec) {
        var sp = Sonatype.lib.Permissions;

        if (rec.data.resourceURI && rec.data.userManaged && sp.checkPermission('nexus:artifact', sp.CREATE) && rec.data.repoType == 'hosted' && rec.data.format && rec.data.format.toUpperCase() == 'NUGET')
        {

          Ext.Ajax.request({
                url : rec.data.resourceURI,
                scope : this,
                callback : function(options, success, response) {
                  if (success)
                  {
                    var statusResp = Ext.decode(response.responseText);
                    if (statusResp.data)
                    {
                      if (statusResp.data.writePolicy == 'ALLOW_WRITE' || statusResp.data.writePolicy == 'ALLOW_WRITE_ONCE')
                      {
                        var uploadPanel = new Sonatype.repoServer.NugetUploadPanel({
                              payload : rec,
                              name : 'upload'
                            });
                        var card = cardPanel.add({
                              xtype : 'panel',
                              layout : 'fit',
                              tabTitle : 'NuPkg Upload',
                              name : 'uploadPanel',
                              items : [uploadPanel]
                            });

                        card.on('show', function(p) {
                              // This is a hack to fix the width of the edit box
                              // in IE
                              p.doLayout();
                              p.find('name', 'filenameField')[0].setValue('.');
                              p.find('name', 'filenameField')[0].setValue('');

                              // another hack to make the whole browse button
                              // clickable
                              if (!p.browseButtonsUpdated)
                              {
                                var b = p.find('xtype', 'browsebutton');
                                for (var i = 0; i < b.length; i++)
                                {
                                  b[i].setClipSize();
                                }
                                p.browseButtonsUpdated = true;
                              }
                            });
                      }
                      else
                      {
                        cardPanel.add({
                              xtype : 'panel',
                              tabTitle : 'NuPkg Upload',
                              name : 'upload',
                              items : [{
                                    border : false,
                                    html : '<div class="little-padding">' + 'Package deployment is disabled for ' + rec.data.name + '.<br /><br />' + 'You can enable it in the "Access Settings" section of the ' + 'repository configuration.</div>'
                                  }]
                            });
                      }

                      cardPanel.doLayout();

                      return;
                    }
                  }
                  Sonatype.utils.connectionError(response, 'There was a problem obtaining repository status.');
                }
              });
        }
      });
});
